/*
Polya Aeppli distribution for the computation of analytical p-values
*/

package algorithms.model;

public class PolyaAeppli
{
	//Lambda parameter for the distribution
	private double lambda;
	//Alpha parameter for the distribution
	private double a;
	
	/* 
	Constructor
	@param a: alpha parameter for the distribution
	@param lambda: lambda parameter for the distribution
	*/
	public PolyaAeppli(double a, double lambda)
	{
		this.a=a;
		this.lambda=lambda;
	}
	
	/*
	Computes the lowertail of the distribution (pvalue = 1 - lowertail)
	@param numOccurrences: number of occurrences of a motif in a target graph
	*/
	public double lowertail(long numOccurrences)
	{
		double z=(1.0-a)*lambda/a;
		double res=0.0;
		if(numOccurrences==0)
			res=Math.exp(-lambda);
		else if(numOccurrences==1)
			res=Math.exp(-lambda)*(1.0+a*z);
		else
		{
			double precVal=Double.MIN_VALUE;
			double currVal=0.0;
			double Lprec=-lambda;
			double Lcour=Lprec+Math.log(a*z);
			double Acour=Lprec;
			double Scour=1.0+a*z; 
			for (int i=2;i<=numOccurrences;i++)
			{
				double Lsuiv=Lcour+Math.log(a*(2.0*i-2.0+z)/i+a*a*(2.0-i)*Math.exp(Lprec-Lcour)/i);
				double Ssuiv=Scour+Math.exp(Lsuiv-Acour);
				double Asuiv=0.0;
				if(Ssuiv>0 && Ssuiv<Double.MAX_VALUE)
					Asuiv=Acour;
				else
				{
					Asuiv=Acour+Math.log(Scour);
					Ssuiv=1.0+Math.exp(Lsuiv-Asuiv); 
				}
				Lprec=Lcour;
				Lcour=Lsuiv; 
				Scour=Ssuiv; 
				Acour=Asuiv;
				currVal=Scour*Math.exp(Acour);
				if(currVal>=1.0)
					break;
				if(currVal>1.0E-322 && currVal==precVal)
					break;
				else
					precVal=currVal;
			} 
			res=currVal;
		}
		return res;
	}

}