/*
Collection of methods for graph analysis
*/

package util;

import java.util.Vector;

public class GraphUtility
{

	/*
	Computes the canonical form of a graph, represented by its adjacency matrix
	@param: the adjacency matrix of a graph
	It returns the adjacency matrix of the graph in canonical form
	*/
	public static boolean[][] getCanonAdj(boolean[][] adjMatrix)
	{
		int numNodes=adjMatrix.length;
		int[] outDegrees=new int[numNodes];
		int[] inDegrees=new int[numNodes];
		int i=0, j=0, k=0;
		Vector<Integer> setAdiacs=new Vector<Integer>();
		for(i=0;i<numNodes;i++)
		{
			setAdiacs.add(i);
			for(j=0;j<numNodes;j++)
			{
				if(adjMatrix[i][j])
				{
					outDegrees[i]++;
					inDegrees[j]++;
				}
			}
		}
		int[] canonMap=new int[numNodes];
		boolean[] used=new boolean[numNodes];
		for(i=0;i<numNodes;i++)
		{
			int maxDegOut=-1;
			int maxDegIn=-1;
			int maxId=0;
			for(j=0;j<setAdiacs.size();j++)
			{
				int idNode=setAdiacs.get(j);
				if(outDegrees[idNode]>maxDegOut)
				{
					maxDegOut=outDegrees[idNode];
					maxDegIn=inDegrees[idNode];
					maxId=idNode;
				}
				else if(outDegrees[idNode]==maxDegOut)
				{
					if(inDegrees[idNode]>maxDegIn)
					{
						maxDegIn=inDegrees[idNode];
						maxId=idNode;
					}
				}
			}
			//System.out.println(maxId);
			canonMap[i]=maxId;
			used[maxId]=true;
			setAdiacs.clear();
			for(j=0;j<=i;j++)
			{
				int idSource=canonMap[j];
				for(k=0;k<numNodes;k++)
				{
					if(adjMatrix[idSource][k] && !used[k] && !setAdiacs.contains(k))
						setAdiacs.add(k);
				}
			}
            if(setAdiacs.isEmpty())
            {
                for(j=0;j<numNodes;j++)
                {
                    if(!used[j])
                        setAdiacs.add(j);
                }
            }
			//System.out.println(setAdiacs);
		}
        boolean[][] finalAdjMatrix=new boolean[numNodes][numNodes];
        for(i=0;i<numNodes;i++)
        {
            for(j=0;j<numNodes;j++)
                finalAdjMatrix[i][j]=adjMatrix[canonMap[i]][canonMap[j]];
        }
        return finalAdjMatrix;
	}

	/*
	Sort an array in descending order using insertion sort algorithm
	*/
	public static void insertionSortDesc(int[] a)
	{
		int i=0, j=0;
		for(i=1;i<a.length;i++)
		{
			int value=a[i];
			j=i-1;
			while(j>=0 && a[j]<value)
			{
				a[j+1]=a[j];
				j=j-1;
			}
			a[j+1]=value;
		}
	}
	
	/* 
	Convert an adjacency string of a graph into an andjacency matrix
	@param str: adjacency string of the graph
	*/
	public static boolean[][] getAdjMatrix(String str)
	{
		int sizeMotif=(int)Math.sqrt(str.length());
		boolean[][] adjMatrix=new boolean[sizeMotif][sizeMotif];
		int i=0;
		for(i=0;i<str.length();i++)
		{
			if(str.charAt(i)=='1')
				adjMatrix[i/sizeMotif][i%sizeMotif]=true;
			else
				adjMatrix[i/sizeMotif][i%sizeMotif]=false;
		}
		return adjMatrix;
	}

	/*
	Convert an adjacency matrix of a graph into an adjacency string
	@param adjMat: adjacency matrix of the graph
	*/
	public static String getAdjString(boolean[][] adjMat)
	{
		String adjString="";
		int i=0, j=0;
		for(i=0;i<adjMat.length;i++)
		{
			for(j=0;j<adjMat.length;j++)
			{
				if(adjMat[i][j])
					adjString+="1";
				else
					adjString+="0";
			}
		}
		return adjString;
	}

}